import set_pathes

import unittest
from momo.prototype import Prototype


class TestPrototype(unittest.TestCase):
    def test_init_prototype(self):
        prototype = Prototype(data=[1, 2, 3], index=['a', 'b', 'c'])
        self.assertEqual(prototype.to_list(), [1, 2, 3])


    def test_set_marks(self):
        prototype = Prototype(data=[1, 2, 3], index=['a', 'b', 'c'])
        prototype.set_marks([4, 5, 6])
        self.assertEqual(prototype.to_list(), [4, 5, 6])


    def test_set_incorect_marks(self):
        prototype = Prototype(data=[1, 2, 3], index=['a', 'b', 'c'])
        with self.assertRaises(ValueError):
            prototype.set_marks([4, 5, 6, 7])




if __name__ == '__main__':
	unittest.main()



import set_pathes

import unittest
from momo.system_models.system_models import SystemModel



class TestSystemModel(unittest.TestCase):
    def setUp(self):
        self.system_model = SystemModel("test_model", [[1, 2, 3], [4, 5, 6], [7, 8, 9]], features=['f1', 'f2', 'f3'], alternatives=["a1", "a2", "a3"])


    def test_init_system_model_incorect_name(self):
        with self.assertRaises(ValueError):
            SystemModel(name=None)


    def test_init_system_model_incorect_features(self):
        with self.assertRaises(ValueError):
            SystemModel(name="test", data=[[1, 2, 3], [4, 5, 6], [7, 8, 9]], features=['f1', 'f2'], alternatives=["a1", "a2", "a3"])


    def test_init_system_model_incorect_alternatives(self):
        with self.assertRaises(ValueError):
            SystemModel(name="test", data=[[1, 2, 3], [4, 5, 6], [7, 8, 9]], features=['f1', 'f2', 'f3'], alternatives=["a1", "a2"])


    def test_init_system_model(self):
        system_model = SystemModel("test")
        self.assertEqual(system_model.name, "test")


    def test_init_system_model(self):
        self.assertEqual(self.system_model.data['a1'].to_list(), [1, 4, 7])


    def test_add_feature(self):
        self.system_model.add_feature('f4', [10, 11, 12])
        self.assertEqual(self.system_model.loc['f4'].to_list(), [10, 11, 12])


    def test_add_feature_incorect_length(self):
        with self.assertRaises(ValueError):
            self.system_model.add_feature('f4', [10, 11])


    def test_add_feature_incorect_name(self):
        with self.assertRaises(ValueError):
            self.system_model.add_feature(None, [10, 11, 12])


    def test_add_alternative(self):
        self.system_model.add_alternative('a4', [13, 14, 15])
        self.assertEqual(self.system_model['a4'].to_list(), [13, 14, 15])


    def test_add_alternative_incorect_length(self):
        with self.assertRaises(ValueError):
            self.system_model.add_alternative('a4', [13, 14])


    def test_add_alternative_incorect_name(self):
        with self.assertRaises(ValueError):
            self.system_model.add_alternative(None, [13, 14, 15])


    def test_remove_alternative(self):
        self.system_model.remove_alternative('a1')
        self.assertNotIn('a1', self.system_model.data.columns)


    def test_remove_alternative_incorect_key(self):
        self.system_model.remove_alternative('a1')
        with self.assertRaises(KeyError):
            self.system_model['a1']


    def test_remove_feature(self):
        self.system_model.remove_feature('f1')
        self.assertNotIn('f1', self.system_model.data.index)


    def test_remove_features_incorect_key(self):
        self.system_model.remove_feature('f1')
        with self.assertRaises(KeyError):
            self.system_model.loc['f1']



if __name__ == "__main__":
    unittest.main()



import sys
import os

current_file_path = os.path.abspath(__file__)
tests_dir = os.path.dirname(current_file_path)
project_root_dir = os.path.dirname(os.path.dirname(tests_dir))
sys.path.append(project_root_dir)



import pandas as pd
import set_pathes

import unittest
from momo.system_models.system_models import SystemModel, MultiSystemModel



class TestMultySystemModel(unittest.TestCase):
    def setUp(self):
        self.dbms = SystemModel("DBMS")
        self.dbms.add_feature("Security", {"MySQL": 1, "MS SQL": 1, "Oracle": 1})
        self.dbms.add_feature("Performance", [1, 1, 1])
        self.dbms.add_feature("Speed", [0, 1, 1])

        self.connector = SystemModel("Connector")
        self.connector.add_feature("Flexibility", {"Copper": 1, "Aluminum": 1})
        self.connector.add_feature("Cost", {"Copper": 1, "Aluminum": 1})


    def test_init_multi_system_model(self):
        multi_system_model = MultiSystemModel([self.dbms])
        self.assertEqual(multi_system_model.systems, {"DBMS": self.dbms})


    def test_init_multi_system_model_with_tuple(self):
        multi_system_model = MultiSystemModel((self.dbms, self.connector))
        self.assertEqual(multi_system_model.systems, {"DBMS": self.dbms, "Connector": self.connector})


    def test_init_multi_system_model_with_set(self):
        multi_system_model = MultiSystemModel({self.dbms, self.connector})
        self.assertEqual(multi_system_model.systems, {"DBMS": self.dbms, "Connector": self.connector})


    def test_init_multi_system_model_incopatible_type(self):
        with self.assertRaises(ValueError):
            MultiSystemModel("DBMS")


    def test_init_multi_system_model_none(self):
        multi_system_model = MultiSystemModel()
        self.assertEqual(multi_system_model.systems, {})


    def test_add_system(self):
        multi_system_model = MultiSystemModel()
        multi_system_model.add_system(self.dbms)
        self.assertEqual(multi_system_model.systems, {"DBMS": self.dbms})


    def test_add_system_incopatible_type(self):
        multi_system_model = MultiSystemModel()
        with self.assertRaises(ValueError):
            multi_system_model.add_system("DBMS")


    def test_add_systems(self):
        multi_system_model = MultiSystemModel()
        multi_system_model.add_systems([self.dbms, self.connector])
        self.assertEqual(multi_system_model.systems, {"DBMS": self.dbms, "Connector": self.connector})


    def test_add_systems_incopatible_type(self):
        multi_system_model = MultiSystemModel()
        with self.assertRaises(ValueError):
            multi_system_model.add_systems("DBMS")


    def test_remove_system(self):
        multi_system_model = MultiSystemModel([self.dbms, self.connector])
        multi_system_model.remove_system("DBMS")
        self.assertEqual(multi_system_model.systems, {"Connector": self.connector})


    def test_remove_system_not_found(self):
        multi_system_model = MultiSystemModel([self.dbms, self.connector])
        with self.assertRaises(ValueError):
            multi_system_model.remove_system("DBMS1")


    def test_get_prototype(self):
        multi_system_model = MultiSystemModel([self.dbms, self.connector])
        prototype = multi_system_model.get_prototype()
        expected_prototype = pd.Series({
            ("DBMS", "Security"): 0,
            ("DBMS", "Performance"): 0,
            ("DBMS", "Speed"): 0,
            ("Connector", "Flexibility"): 0,
            ("Connector", "Cost"): 0,
        })

        self.assertTrue(prototype.equals(expected_prototype))


    def test_get_prototype_with_one_system(self):
        multi_system_model = MultiSystemModel([self.dbms])
        prototype = multi_system_model.get_prototype()
        expected_prototype = pd.Series({
            ("DBMS", "Security"): 0,
            ("DBMS", "Performance"): 0,
            ("DBMS", "Speed"): 0,
        })

        self.assertTrue(prototype.equals(expected_prototype))


    def test_get_prototype_with_empty_systems(self):
        multi_system_model = MultiSystemModel()
        prototype = multi_system_model.get_prototype()
        expected_prototype = pd.Series({})

        self.assertTrue(prototype.equals(expected_prototype))


    def test_get_all_combinations(self):
        multi_system_model = MultiSystemModel([self.dbms, self.connector])
        combinations = multi_system_model.get_all_combinations()
        expected_combinations = pd.DataFrame(data={
            ("DBMS", "Security"):         [1, 1, 1, 1, 1, 1],
            ("DBMS", "Performance"):      [1, 1, 1, 1, 1, 1],
            ("DBMS", "Speed"):            [0, 0, 1, 1, 1, 1],
            ("Connector", "Flexibility"): [1, 1, 1, 1, 1, 1],
            ("Connector", "Cost"):        [1, 1, 1, 1, 1, 1],
        }, index=[('MySQL', 'Copper'), ('MySQL', 'Aluminum'),
                ('MS SQL', 'Copper'), ('MS SQL', 'Aluminum'),
                ('Oracle', 'Copper'), ('Oracle', 'Aluminum'),])

        self.assertTrue(combinations.equals(expected_combinations.transpose()))



if __name__ == "__main__":
    unittest.main()



import pandas as pd
from itertools import product
from momo.prototype import Prototype
from momo.system_models.__item_apender import _FeaturesAppender, _AlternativesAppender


class SystemModel:
    """Represents a system model."""

    def __init__(self, name: str, data=None, features=None, alternatives=None):
        """
        Initializes a SystemModel instance.

        Args:
            name (str): The name of the system model.
            data (list or None): The data for the system model.
            features (list or None): The list of features for the system model.
            alternatives (list or None): The list of alternatives for the system model.
        """
        self._init_name(name)
        self._validate_as_list_string(features, "features")
        self._validate_as_list_string(alternatives, "alternatives")

        self._validate_meanshures_of_data_features_alternatives(data, features, alternatives)

        self.data = pd.DataFrame(data=data, index=features, columns=alternatives)
        self.__features_appender = _FeaturesAppender(self.data)
        self.__alternatives_appender = _AlternativesAppender(self.data)


    def _init_name(self, name):
        """
        Initializes the name attribute of the SystemModel instance.

        Args:
            name: The name of the system model.

        Raises:
            ValueError: If the name parameter is not a string.
        """
        if not isinstance(name, str):
            raise ValueError("The name parameter must be a string.")
        self.name = name


    def _validate_meanshures_of_data_features_alternatives(self, data, features, alternatives):
        """
        Validates the dimensions of the data, features, and alternatives.

        Args:
            data: The data for the system model.
            features: The list of features for the system model.
            alternatives: The list of alternatives for the system model.

        Raises:
            ValueError: If the dimensions of the data, features, and alternatives do not match.
        """
        if data is not None:
            if features is not None and len(data) != len(features):
                raise ValueError("The number of rows in data must match the length of features.")
            if alternatives is not None and len(data[0]) != len(alternatives):
                raise ValueError("The number of columns in data must match the length of alternatives.")


    def _validate_as_list_string(self, value, name):
        """
        Validates that a value is a list of strings.

        Args:
            value: The value to validate.
            name: The name of the value.

        Raises:
            ValueError: If the value is not a list of strings.
        """
        if value is not None and not all(isinstance(v, str) for v in value):
            raise ValueError(f"All elements of {name} must be strings.")


    def _validate_as(self, value, name, expected_type):
        """
        Validates that a value is of the expected type.

        Args:
            value: The value to validate.
            name: The name of the value.
            expected_type: The expected type of the value.

        Raises:
            ValueError: If the value is not of the expected type.
        """
        if not isinstance(value, expected_type):
            raise ValueError(f"The {name} parameter must be of type {expected_type.__name__}.")


    def _validate_length(self, value, name, expected_length):
        """
        Validates the length of a value.

        Args:
            value: The value to validate.
            name: The name of the value.
            expected_length: The expected length of the value.

        Raises:
            ValueError: If the value does not have the expected length.
        """
        if value is not None and len(value) != expected_length:
            raise ValueError(f"The {name} parameter must have a length of {expected_length}.")



    def add_feature(self, feature_name: str, alternatives: list | dict):
        """
        Adds a feature to the system model.

        Args:
            feature_name (str): The name of the feature.
            alternatives (list or dict): The list or dictionary of alternatives for the feature.
        """
        self.__features_appender.data_frame = self.data  # Refresh the pointed data.
        self.data = self.__features_appender.add_item(feature_name, alternatives)


    def add_alternative(self, alternative_name, features : list | dict):
        """
        Adds an alternative to the system model.

        Args:
            alternative_name: The name of the alternative.
            features (list or dict): The list or dictionary of features for the alternative.
        """
        self.__alternatives_appender.data_frame = self.data  # Refresh the pointed data.
        self.data = self.__alternatives_appender.add_item(alternative_name, features)


    def remove_feature(self, feature_name):
        """
        Removes a feature from the system model.

        Args:
            feature_name: The name of the feature to remove.

        Raises:
            ValueError: If the feature does not exist.
        """
        self._validate_as(feature_name, "feature_name", str)

        if feature_name not in self.data.index:
            raise ValueError(f"Feature '{feature_name}' does not exist.")

        self.data = self.data.drop(index=feature_name)


    def remove_alternative(self, alternative_name):
        """
        Removes an alternative from the system model.

        Args:
            alternative_name: The name of the alternative to remove.

        Raises:
            ValueError: If the alternative does not exist.
        """
        self._validate_as(alternative_name, "alternative_name", str)

        if alternative_name not in self.data.columns:
            raise ValueError(f"Alternative '{alternative_name}' does not exist.")

        self.data = self.data.drop(columns=alternative_name)



    def get_features(self):
        """
        Returns a tuple of the features in the system model.

        Returns:
            tuple: The features in the system model.
        """
        return tuple(self.data.index)

    @property
    def features(self):
        """
        Returns the features property of the system model's data.

        Returns:
            pandas.DataFrame.index: The features property of the system model's data.
        """
        return self.get_features()


    def get_alternatives(self):
        """
        Returns a tuple of the alternatives in the system model.

        Returns:
            tuple: The alternatives in the system model.
        """
        return tuple(self.data.columns)

    @property
    def alternatives(self):
        """
        Returns the alternatives property of the system model's data.

        Returns:
            pandas.DataFrame.columns: The alternatives property of the system model's data.
        """
        return self.get_alternatives()

    @property
    def loc(self):
        """
        Returns the loc property of the system model's data.

        Returns:
            pandas.DataFrame.loc: The loc property of the system model's data.
        """
        return self.data.loc


    def __getitem__(self, key):
        """
        Returns the value at the specified key in the system model's data.

        Args:
            key: The key to retrieve the value for.

        Returns:
            Any: The value at the specified key in the system model's data.
        """
        return self.data[key]


    def __setitem__(self, key, value):
        """
        Sets the value at the specified key in the system model's data.

        Args:
            key: The key to set the value for.
            value: The value to set.
        """
        self.data[key] = value


    def __str__(self):
        """
        Returns a string representation of the system model.

        Returns:
            str: A string representation of the system model.
        """
        return f'"{self.name}"\n{self.data}'



class MultiSystemModel:
    """Represents a multi-system model."""

    def __init__(self, system_models: list | tuple | set | None = None):
        """
        Initializes a MultiSystemModel instance.

        Args:
            system_models (list or tuple or set or None): The list, tuple, or set of system models.
        """
        self.systems = {}

        if isinstance(system_models, MultiSystemModel):
            self.systems = system_models.systems
        elif isinstance(system_models, list):
            self._constructor_from_list(system_models)
        elif isinstance(system_models, tuple):
            self._constructor_from_tuple(system_models)
        elif isinstance(system_models, set):
            self._constructor_from_set(system_models)
        elif system_models is None:
            pass # Do nothing
        else:
            raise ValueError("The system_models parameter must be a list, tuple or set.")


    def _constructor_from_list(self, system_models):
        """
        Constructs the MultiSystemModel instance from a list of system models.

        Args:
            system_models (list): The list of system models.

        Raises:
            ValueError: If any element in the list is not a SystemModel instance.
        """
        if all(isinstance(system, SystemModel) for system in system_models):
            for system in system_models:
                self.add_system(system)
        else:
            raise ValueError("All elements in the list must be SystemModel instances.")


    def _constructor_from_tuple(self, system_models):
        """
        Constructs the MultiSystemModel instance from a tuple of system models.

        Args:
            system_models (tuple): The tuple of system models.

        Raises:
            ValueError: If any element in the tuple is not a SystemModel instance.
        """
        if all(isinstance(system, SystemModel) for system in system_models):
            for system in system_models:
                self.add_system(system)
        else:
            raise ValueError("All elements in the tuple must be SystemModel instances.")


    def _constructor_from_set(self, system_models):
        """
        Constructs the MultiSystemModel instance from a set of system models.

        Args:
            system_models (set): The set of system models.

        Raises:
            ValueError: If any element in the set is not a SystemModel instance.
        """
        if all(isinstance(system, SystemModel) for system in system_models):
            for system in system_models:
                self.add_system(system)
        else:
            raise ValueError("All elements in the set must be SystemModel instances.")


    def add_system(self, system_model: SystemModel):
        """
        Adds a system model to the multi-system model.

        Args:
            system_model (SystemModel): The system model to add.

        Raises:
            ValueError: If the system_model parameter is not a SystemModel instance.
        """
        if not isinstance(system_model, SystemModel):
            raise ValueError("The system_model parameter must be a SystemModel instance.")

        self.systems[system_model.name] = system_model


    def add_systems(self, system_models: list | tuple | set):
        """
        Adds multiple system models to the multi-system model.

        Args:
            system_models (list or tuple or set): The list, tuple, or set of system models.

        Raises:
            ValueError: If the system_models parameter is not a list, tuple, or set.
        """
        if not isinstance(system_models, (list, tuple, set)):
            raise ValueError("The system_models parameter must be a list, tuple or set.")

        for system in system_models:
            self.add_system(system)


    def remove_system(self, system_name):
        """
        Removes a system model from the multi-system model.

        Args:
            system_name: The name of the system model to remove.

        Raises:
            ValueError: If the system model does not exist.
        """
        if self.__has_system(system_name):
            del self.systems[system_name]
        else:
            raise ValueError(f"System '{system_name}' does not exist.")


    def __has_system(self, system_name):
        """
        Checks if a system model exists in the multi-system model.

        Args:
            system_name: The name of the system model to check.

        Returns:
            bool: True if the system model exists, False otherwise.
        """
        return system_name in self.systems


    def get_system_names(self):
        """
        Returns a tuple of the names of the system models in the multi-system model.

        Returns:
            tuple: The names of the system models.
        """
        return tuple(self.systems.keys())


    def get_all_combinations(self):
        """
        Returns a DataFrame containing all combinations of alternatives from the system models.

        Returns:
            pandas.DataFrame: The DataFrame containing all combinations of alternatives.
        """
        system_names = list(self.systems.keys())
        system_data = [self.systems[name].data for name in system_names]

        related_features = self.get_features_related_to_system()
        alternatives_name = list(product(*[data.columns for data in system_data]))

        result_df = pd.DataFrame(index=related_features)

        # Fill the DataFrame with combinations
        for combination in alternatives_name:
            result_df[combination] = 0

            for system_name, system_df in zip(system_names, system_data):
                for feature in system_df.index:
                    if related_feature := self.__get_related_feature(feature, related_features):
                        value = system_df.loc[feature, combination[system_names.index(system_name)]]
                        result_df.at[related_feature, combination] = value

        return result_df


    def __get_related_feature(self, feature, related_features):
        """
        Returns the related feature for a given feature.

        Args:
            feature: The feature to find the related feature for.
            related_features: The tuple of related features.

        Returns:
            str: The related feature.

        Raises:
            ValueError: If the feature is not related to any system.
        """
        for related_feature in related_features:
            if feature in related_feature:
                return related_feature

        raise ValueError(f"Feature {feature} is not related to any system.")


    def get_features_related_to_system(self):
        """
        Returns a tuple of features related to each system in the multi-system model.

        Returns:
            tuple: The features related to each system.
        """
        systems = self.get_system_names()
        features = [self.systems[name].data.index for name in systems]

        related_features = []

        for system, feature_list in zip(systems, features):
            for feature in feature_list:
                    related_features.append((system, feature))

        return tuple(related_features)


    def get_all_features(self):
        """
        Returns a tuple of all features in the multi-system model.

        Returns:
            tuple: All features in the multi-system model.
        """
        all_features = list()
        for system_data in self.systems.values():
            all_features.extend(system_data.data.index.to_list())
        return tuple(all_features)


    def get_prototype(self):
        """
        Returns a Prototype instance based on the multi-system model.

        Returns:
            Prototype: The Prototype instance.
        """
        if not self.systems:
            return Prototype()

        related_features = self.get_features_related_to_system()
        prototype = Prototype(data=[0 for _ in related_features], index=related_features)
        return prototype
    #
    # =====================================================================

    def __str__(self):
        """
        Returns a string representation of the multi-system model.

        Returns:
            str: A string representation of the multi-system model.
        """
        return self.get_all_combinations().__str__()



from .prototype import Prototype
from .system_models.system_models import MultiSystemModel


class MoMoModel:
    """
    Represents a MoMo (Multi-Object Multi-System) model.

    Args:
        system_models (MultiSystemModel | list | tuple | set): The system models used in the MoMo model.
        prototype (Prototype | None, optional): The prototype object. Defaults to None.
    """

    def __init__(
        self,
        system_models: MultiSystemModel | list | tuple | set,
        prototype: Prototype | None = None,
    ):
        """
        Initializes a MoMoModel instance.

        Args:
            system_models (MultiSystemModel | list | tuple | set): The system models used in the MoMo model.
            prototype (Prototype | None, optional): The prototype object. Defaults to None.
        """
        self.u = 0
        self.system_models = MultiSystemModel(system_models)
        self._init_prototype(prototype)

    def _init_prototype(self, value):
        """
        Initializes the prototype object.

        Args:
            value (Prototype | None): The prototype object or None.

        Raises:
            ValueError: If the prototype is not a Prototype or None instance.
            ValueError: If the prototype does not have the same index as the system models.
        """
        if value is None:
            self.prototype = self.system_models.get_prototype()
        elif isinstance(value, Prototype):
            if value.index.equals(self.system_models.get_prototype().index):
                self.prototype = value
            else:
                raise ValueError("The prototype must have the same index as the system models.")
        else:
            raise ValueError("The prototype must be a Prototype or None instance.")

    def get_prototype(self):
        """
        Returns the prototype object.

        Returns:
            Prototype: The prototype object.
        """
        return self.prototype

    def set_prototype(self, prototype):
        """
        Sets the prototype object.

        Args:
            prototype (Prototype): The prototype object.
        """
        self._init_prototype(prototype)

    @property
    def prototype_(self):
        """
        Prototype: The prototype object.
        """
        return self.get_prototype()

    @prototype_.setter
    def prototype_(self, prototype):
        self.set_prototype(prototype)

    def get_system_models(self):
        """
        Returns the system models.

        Returns:
            MultiSystemModel: The system models.
        """
        return self.system_models

    def set_system_models(self, system_models):
        """
        Sets the system models.

        Args:
            system_models (MultiSystemModel | list | tuple | set): The system models.
        """
        self.system_models = MultiSystemModel(system_models)
        self.prototype = self.system_models.get_prototype()

    @property
    def system_models_(self):
        """
        MultiSystemModel: The system models.
        """
        return self.get_system_models()

    @system_models_.setter
    def system_models_(self, system_models):
        self.set_system_models(system_models)


    def __calculate_similarity_measures(self, prototype, compared_system):
        """
        Calculates the similarity measures between the prototype and a compared system.

        Args:
            prototype (Prototype): The prototype object.
            compared_system (Series): The compared system.

        Returns:
            float: The similarity measure.
        """
        intersection_num = (prototype == compared_system).sum()
        card_prototype = prototype.sum()
        card_compared_system = compared_system.sum()
        u = self.u

        return 2 * intersection_num / ((1 + u) * (card_prototype + card_compared_system) - 2 * u * intersection_num)

    def get_similarity_measures(self):
        """
        Calculates the similarity measures between the prototype and all combinations of system models.

        Returns:
            dict: A dictionary containing the similarity measures for each combination.
        """
        similarity_measures = {}
        combinations = self.system_models.get_all_combinations()

        for combination in combinations.columns:
            similarity_measures[combination] = self.__calculate_similarity_measures(
                self.prototype, combinations[combination]
            )

        return similarity_measures


    def __str__(self):
        """
        Returns a string representation of the MoMoModel object.

        Returns:
            str: A string representation of the MoMoModel object.
        """
        output = f"Prototype:\n{self.prototype}\n\n"
        output += f"System Models:\n{self.system_models}"

        return output



import pandas as pd


class Prototype(pd.Series):
    """
    A custom subclass of pd.Series representing a prototype.

    Parameters:
    - data (array-like, Iterable, dict, or scalar): The data to be stored in the prototype.
    - index (array-like or Index (1d)): The index labels associated with the data.

    Inherits all the functionality of pd.Series.
    """

    def __init__(self, data=None, index=None):
        """
        Initialize a new Prototype object.

        Parameters:
        - data (array-like, Iterable, dict, or scalar): The data to be stored in the prototype.
        - index (array-like or Index (1d)): The index labels associated with the data.
        """
        super().__init__(data=data, index=index)

    def set_marks(self, marks_list: dict | list):
        """
        Set the marks of the prototype.

        Parameters:
        - marks_list (array-like, Iterable, dict): The marks to be set.

        Returns:
        None
        """
        if isinstance(marks_list, dict):
            self._set_marks_dict(marks_list)
        elif isinstance(marks_list, list):
            self._set_marks_list(marks_list)
        else:
            raise ValueError("The marks_list parameter must be a dictionary or a list.")


    def _set_marks_dict(self, marks_dict: dict):
        """
        Set the marks of the prototype from a dictionary.

        Parameters:
        - marks_dict (dict): The marks to be set.

        Returns:
        None
        """
        for outer_key, inner_dict in marks_dict.items():
            for inner_key, value in inner_dict.items():
                self[(outer_key, inner_key)] = value


    def _set_marks_list(self, marks_list: list):
        """
        Set the marks of the prototype from a list.

        Parameters:
        - marks_list (list): The marks to be set.

        Returns:
        None
        """
        self[:] = marks_list



import set_pathes

import unittest
from momo.model import MoMoModel
from momo.prototype import Prototype
from momo.system_models.system_models import MultiSystemModel, SystemModel


class TestMoMoModel(unittest.TestCase):
    def setUp(self):
        dbms = SystemModel("DBMS")
        dbms.add_feature("Security", {"MySQL": 1, "MS SQL": 1, "Oracle": 1})
        dbms.add_feature("Performance", [1, 1, 1])
        dbms.add_feature("Speed", [0, 1, 1])

        connector = SystemModel("Connector")
        connector.add_feature("Flexibility", {"Copper": 1, "Aluminum": 1})
        connector.add_feature("Cost", {"Copper": 1, "Aluminum": 1})

        self.multy_system = MultiSystemModel([dbms, connector])
        self.model = MoMoModel(self.multy_system)


    def test_init(self):
        self.assertEqual(self.model.u, 0)
        self.assertDictEqual(self.model.system_models.systems, self.multy_system.systems)
        self.assertTrue(self.model.prototype.equals(self.multy_system.get_prototype()))


    def test_init_model_incorect_system_models(self):
        with self.assertRaises(ValueError):
            MoMoModel([1, 2, 3])


    def test_init_model_incorect_prototype(self):
        with self.assertRaises(ValueError):
            MoMoModel(self.multy_system, 1)


    def test_init_model_incorect_meanshure_of_prototype(self):
        with self.assertRaises(ValueError):
            MoMoModel(self.multy_system, Prototype([0, 0, 0], ["Security", "Performance", "Speed"]))


    def test_init_model_with_prototype(self):
        expected_prototype = Prototype([0, 0, 0], [("DBMS", "Security"), ("DBMS", "Performance"), ("DBMS", "Speed")])
        actual_prototype = MoMoModel([self.multy_system.systems['DBMS']]).get_prototype()
        self.assertTrue(expected_prototype.equals(actual_prototype))


    def test_get_similarity_measures(self):
        self.model.get_similarity_measures()
        self.assertTrue(True)



if __name__ == '__main__':
	unittest.main()



from abc import ABC, abstractmethod
import pandas as pd


class _ItemAppender(ABC):
    """
    Abstract base class for appending items to a data frame.
    """

    def __init__(self, data_frame):
        """
        Initializes the _ItemAppender object.

        Args:
            data_frame (pandas.DataFrame): The data frame to append items to.
        """
        self.data_frame = data_frame

    def add_item(self, item_name, item_values):
        """
        Adds an item to the data frame.

        Args:
            item_name (str): The name of the item.
            item_values (list or dict): The values of the item.

        Returns:
            pandas.DataFrame: The updated data frame.
        """
        self._check_item_name(item_name)
        self._check_item_values(item_name, item_values)
        return self.data_frame

    def _check_item_name(self, item_name):
        """
        Checks if the item name is a string.

        Args:
            item_name (str): The name of the item.

        Raises:
            ValueError: If the item name is not a string.
        """
        if not isinstance(item_name, str):
            raise ValueError("The item name must be a string.")

    def _check_item_values(self, item_name, item_values):
        """
        Checks if the item values are a list or a dictionary.

        Args:
            item_name (str): The name of the item.
            item_values (list or dict): The values of the item.

        Raises:
            ValueError: If the item values are not a list or a dictionary.
        """
        if isinstance(item_values, list):
            self._add_list(item_name, item_values)
        elif isinstance(item_values, dict):
            self._add_dict(item_name, item_values)
        else:
            raise ValueError("The item values must be a list or a dictionary.")

    def _check_length_dict(self, left, right, what):
        """
        Checks if the length of the item values is less than or equal to the number of alternatives or features.

        Args:
            left (list): The left side of the comparison.
            right (list): The right side of the comparison.
            what (str): The type of values being compared (alternatives or features).

        Raises:
            ValueError: If the length of the item values is greater than the number of alternatives or features.
        """
        if len(left) > len(right) and len(right):
            raise ValueError(f"The length of the item values must be less or equal to the number of {what}.")

    def _check_keys(self, keys, right):
        """
        Checks if the keys in the item values are valid.

        Args:
            keys (list): The keys in the item values.
            right (list): The valid keys.

        Raises:
            ValueError: If any key in the item values is not a valid key.
        """
        if not len(right):
            return

        for key in keys:
            if key not in right:
                raise ValueError(f"The key '{key}' is not a valid key.")

    @abstractmethod
    def _add_list(self, item_name, item_values):
        """
        Abstract method for adding item values as a list.

        Args:
            item_name (str): The name of the item.
            item_values (list): The values of the item.
        """
        pass

    @abstractmethod
    def _add_dict(self, item_name, item_values):
        """
        Abstract method for adding item values as a dictionary.

        Args:
            item_name (str): The name of the item.
            item_values (dict): The values of the item.
        """
        pass


class _FeaturesAppender(_ItemAppender):
    """
    Class for appending features to a data frame.
    """

    def _add_list(self, item_name, item_values):
        """
        Adds feature values as a list to the data frame.

        Args:
            item_name (str): The name of the feature.
            item_values (list): The values of the feature.

        Raises:
            ValueError: If the length of the item values is not equal to the number of alternatives.
        """
        if len(item_values) != len(self.data_frame.columns):
            raise ValueError("The length of the item values must be equal to the number of alternatives.")

        row = pd.DataFrame(data=[item_values], index=[item_name], columns=self.data_frame.columns)
        self.data_frame = pd.concat([self.data_frame, row], axis=0)

    def _add_dict(self, item_name, item_values):
        """
        Adds feature values as a dictionary to the data frame.

        Args:
            item_name (str): The name of the feature.
            item_values (dict): The values of the feature.

        Raises:
            ValueError: If the length of the item values is not less than or equal to the number of alternatives.
            ValueError: If any key in the item values is not a valid key.
        """
        self._check_length_dict(list(item_values.keys()), self.data_frame.columns.to_list(), "alternatives")
        self._check_keys(item_values.keys(), self.data_frame.columns)

        if not self.data_frame.empty:
            item_values = {key: item_values.get(key, 0) for key in self.data_frame.columns}

        row = pd.DataFrame(item_values, index=[item_name])
        self.data_frame = pd.concat([self.data_frame, row], axis=0)


class _AlternativesAppender(_ItemAppender):
    """
    Class for appending alternatives to a data frame.
    """

    def _add_list(self, item_name, item_values):
        """
        Adds alternative values as a list to the data frame.

        Args:
            item_name (str): The name of the alternative.
            item_values (list): The values of the alternative.

        Raises:
            ValueError: If the length of the item values is not equal to the number of features.
        """
        if len(item_values) != len(self.data_frame.index.to_list()):
            raise ValueError("The length of the item values must be equal to the number of features.")

        column = pd.Series(data=item_values, index=self.data_frame.index)
        self.data_frame[item_name] = column

    def _add_dict(self, item_name, item_values):
        """
        Adds alternative values as a dictionary to the data frame.

        Args:
            item_name (str): The name of the alternative.
            item_values (dict): The values of the alternative.

        Raises:
            ValueError: If the length of the item values is not less than or equal to the number of features.
            ValueError: If any key in the item values is not a valid key.
        """
        self._check_length_dict(list(item_values.keys()), self.data_frame.index.to_list(), "features")
        self._check_keys(item_values.keys(), self.data_frame.index)

        if not self.data_frame.empty:
            item_values = {key: item_values.get(key, 0) for key in self.data_frame.index}

        self.data_frame[item_name] = item_values



from setuptools import setup, find_packages

setup(
    name='momo-morphological-modeling',
    version='1.1.0',
    description='MoMo is a module that does Morphological Modeling',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Oleh Danylevych',
    author_email='danylevych123@email.com',
    url='https://github.com/danylevych/MoMo',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
